@@ -12,6 +12,8 @@ from urllib.parse import quote
 
 from bs4 import BeautifulSoup
 from django.conf import settings
+from django.contrib import messages
+from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.models import ContentType
 from django.test import TestCase, LiveServerTestCase
 from django.test.testcases import to_list
@@ -23,11 +25,13 @@ from django.utils.timezone import utc
 from django.utils.translation import gettext_lazy as _
 from freezegun import freeze_time
 
-from .factories import SuperUserFactory
+from .factories import AttachmentFactory, SuperUserFactory, UserFactory
 from ..forms import MapEntityForm
 from ..helpers import smart_urljoin
 from ..settings import app_settings
 
+from paperclip.settings import get_attachment_model
+
 
 class AdjustDebugLevel:
     def __init__(self, name, level):
@@ -47,11 +51,10 @@ class MapEntityTest(TestCase):
     model = None
     modelfactory = None
     userfactory = None
-    expected_json_geom = {}
     maxDiff = None
     user = None
 
-    def get_expected_json_attrs(self):
+    def get_expected_geojson_geom(self):
         return {}
 
     def get_expected_geojson_attrs(self):
@@ -103,7 +106,7 @@ class MapEntityTest(TestCase):
         # Make sure database is not empty for this model
         self.modelfactory.create_batch(30)
 
-        response = self.client.get(self.model.get_layer_url())
+        response = self.client.get(self.model.get_layer_list_url())
         self.assertEqual(response.status_code, 200)
         response = self.client.get(self.model.get_datatablelist_url())
         self.assertEqual(response.status_code, 200)
@@ -229,6 +232,58 @@ class MapEntityTest(TestCase):
         else:
             self.assertIn(b'.modifiable = false;', response.content)
 
+    def test_duplicate(self):
+        if self.model is None or not self.model.can_duplicate:
+            return  # Abstract test should not run
+        user = UserFactory()
+        obj = self.modelfactory.create()
+        for perm in Permission.objects.exclude(codename=f'add_{obj._meta.model_name}'):
+            user.user_permissions.add(perm)
+        self.client.force_login(user=user)
+
+        AttachmentFactory.create(content_object=obj, title='attachment')
+
+        self.assertEqual(obj._meta.model.objects.count(), 1)
+        self.assertEqual(get_attachment_model().objects.count(), 1)
+
+        response = self.client.post(obj.get_duplicate_url())
+        self.assertEqual(response.status_code, 403)
+        self.assertEqual(obj._meta.model.objects.count(), 1)
+        self.assertEqual(get_attachment_model().objects.count(), 1)
+
+        user.user_permissions.add(Permission.objects.get(codename=f'add_{obj._meta.model_name}'))
+        self.client.force_login(user=user)
+
+        response = self.client.post(obj.get_duplicate_url())
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(obj._meta.model.objects.count(), 2)
+        self.assertEqual(get_attachment_model().objects.count(), 2)
+
+        msg = [str(message) for message in messages.get_messages(response.wsgi_request)]
+        self.assertIn(f"{self.model._meta.verbose_name} has been duplicated successfully", msg)
+
+        with patch('mapentity.models.DuplicateMixin.duplicate') as mocked:
+            mocked.side_effect = Exception('Error')
+            response = self.client.post(obj.get_duplicate_url())
+
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(obj._meta.model.objects.count(), 2)
+        self.assertEqual(get_attachment_model().objects.count(), 2)
+
+        msg = [str(message) for message in messages.get_messages(response.wsgi_request)]
+        self.assertIn("An error occurred during duplication", msg)
+
+        with patch('mapentity.models.DuplicateMixin.duplicate') as mocked:
+            mocked.return_value = None
+            response = self.client.post(obj.get_duplicate_url())
+
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(obj._meta.model.objects.count(), 2)
+        self.assertEqual(get_attachment_model().objects.count(), 2)
+
+        msg = [str(message) for message in messages.get_messages(response.wsgi_request)]
+        self.assertIn("An error occurred during duplication", msg)
+
     def test_crud_status(self):
         if self.model is None:
             return  # Abstract test should not run
@@ -238,12 +293,18 @@ class MapEntityTest(TestCase):
         response = self.client.get(obj.get_list_url())
         self.assertEqual(response.status_code, 200)
 
+        response = self.client.get(obj.get_layer_list_url())
+        self.assertEqual(response.status_code, 200)
+
         response = self.client.get(obj.get_detail_url().replace(str(obj.pk), '1234567890'))
         self.assertEqual(response.status_code, 404)
 
         response = self.client.get(obj.get_detail_url())
         self.assertEqual(response.status_code, 200)
 
+        response = self.client.get(obj.get_layer_detail_url())
+        self.assertEqual(response.status_code, 200)
+
         response = self.client.get(obj.get_update_url())
         self.assertEqual(response.status_code, 200)
         self._post_update_form(obj)
@@ -294,12 +355,54 @@ class MapEntityTest(TestCase):
                                         'recordsFiltered': 1,
                                         'recordsTotal': 1})
 
+    @freeze_time("2020-03-17")
+    def test_api_geojson_detail_for_model(self):
+        if self.get_expected_geojson_attrs is None:
+            return
+        if self.model is None:
+            return  # Abstract test should not run
+
+        self.obj = self.modelfactory.create()
+        detail_url = '/api/{modelname}/drf/{modelname}s/{id}.geojson'.format(modelname=self.model._meta.model_name,
+                                                                             id=self.obj.pk)
+        response = self.client.get(detail_url)
+        self.assertEqual(response.status_code, 200, f"{detail_url} not found")
+        content_json = response.json()
+        self.assertEqual(content_json, {
+            'type': 'Feature',
+            'geometry': self.get_expected_geojson_geom(),
+            'properties': self.get_expected_geojson_attrs(),
+        })
+
+    @freeze_time("2020-03-17")
+    def test_api_geojson_list_for_model(self):
+        if self.get_expected_geojson_attrs is None:
+            return
+        if self.model is None:
+            return  # Abstract test should not run
+
+        self.obj = self.modelfactory.create()
+        list_url = '/api/{modelname}/drf/{modelname}s.geojson'.format(modelname=self.model._meta.model_name)
+        response = self.client.get(list_url)
+        self.assertEqual(response.status_code, 200, f"{list_url} not found")
+        content_json = response.json()
+        self.assertEqual(content_json, {
+            'type': 'FeatureCollection',
+            'model': f'{self.obj._meta.app_label}.{self.obj._meta.model_name}',
+            'features': [{
+                'type': 'Feature',
+                'geometry': self.get_expected_geojson_geom(),
+                'properties': self.get_expected_geojson_attrs(),
+            }],
+        })
+
 
 @override_settings(MEDIA_ROOT='/tmp/mapentity-media')
 class MapEntityLiveTest(LiveServerTestCase):
     model = None
     userfactory = None
     modelfactory = None
+    geom = 'POINT(0 0)'
 
     def setUp(self):
         app_settings['SENDFILE_HTTP_HEADER'] = None
@@ -335,7 +438,7 @@ class MapEntityLiveTest(LiveServerTestCase):
         latest_updated.return_value = datetime.utcnow().replace(tzinfo=utc)
 
         latest = self.model.latest_updated()
-        geojson_layer_url = self.url_for(self.model.get_layer_url())
+        geojson_layer_url = self.url_for(self.model.get_layer_list_url())
 
         response_1 = self.client.get(geojson_layer_url, allow_redirects=False)
         self.assertEqual(response_1.status_code, 200)
@@ -391,7 +494,7 @@ class MapEntityLiveTest(LiveServerTestCase):
         SuperUserFactory.create(username='Superuser', password='booh')
         self.client.login(username='Superuser', password='booh')
 
-        obj = self.modelfactory.create(geom='POINT(0 0)')
+        obj = self.modelfactory.create(geom=self.geom)
 
         # Initially, map image does not exists
         image_path = obj.get_map_image_path()
@@ -417,7 +520,7 @@ class MapEntityLiveTest(LiveServerTestCase):
         if self.model is None:
             return  # Abstract test should not run
 
-        obj = self.modelfactory.create(geom='POINT(0 0)')
+        obj = self.modelfactory.create(geom=self.geom)
 
         # Mock Screenshot response
         mock_requests.get.return_value.status_code = 200
